
# coding: utf-8

# Square wave generator for MECSim
# =======
# 
# The aim here is to mimic a square wave based on an input dc ramp from E_start to E_rev with a given scan rate. The ramp will be linear from E_start to E_rev and back to E_start with these parameters changed along with the scan rate.
# 
# Output from script is the N_ac signals to add to the ``Master.inp`` file to perform this hack. MECSim is not designed for square waves as the function has no derivatives at the edges of the square wave, so a smooth version must be used. This notebook will output these terms as well as a PNG image file.
# 

# Parameters used for square wave(s)
# -----

# In[1]:

E_start = 1.5    # Starting voltage (V)
E_rev = 0.5     # reverse voltage (V)
n_cycles = 1     # number of cycles of E_start to E_rev
vscan = 1.0      # V/s
N_squares = 2     # number of square waves
N_ac = 100        # number of AC sinusoids to add to make ramp look like a square wave


# **Import python packages as define some functions**

# In[2]:

import numpy as np
import matplotlib.pyplot as plt


# In[3]:

def ConstSquareWave(Amp = 1, T = 1, nres = 10):
    a = np.zeros(nres)
    f = np.zeros(nres)
    for i in range(0,nres):
        n = i*2+1
        a[i] = 4.0/(np.pi * n)*1000 # output in mV
        f[i] = n / (2.0 * T)
    return a, f


# In[4]:

def EvaluateEAC(ACAmp, ACFreq, time):
    x = ACAmp*1.0e-3*np.sin(2.0*np.pi*ACFreq*time)
    return x
def EvaluateRamp(Es, Er, vs, time):
    DE = Es - Er
    T = abs(DE) / vs
    if(time < T):
        x = Es - DE*time/T
    else:
        x = Er + DE*(time - T)/T
    return x


# In[5]:

def PrintSquareWave(a,f,nres=10):
    print "Copy the following lines into Master.inp"
    print nres,"    ! number of species (need n lines below)"
    for i in range(0,nres):
        print a[i],", ",f[i],"  ! AC sin wave: amp (mV), freq(Hz) (REPEAT)"
    return


# **Re-calculate E_start (= E_end), E_rev and the scan rate**
# 
# Minor change in these inputs is required to get a clean fit.

# In[6]:

E_mid = 0.5*( E_start + E_rev )
DE = E_start - E_rev
T = n_cycles * DE / vscan
amp, freq = ConstSquareWave(DE, T/N_squares, N_ac)
DE_new = 1.0e-6
E_start_new = E_mid + DE_new/2.0
E_rev_new = E_mid - DE_new/2.0
vscan_new = DE_new / T
period_square = 2*T #/ N_squares


# **Plot the resulting approximate square wave to file**
# 
# More accurate the larger the number of ac sinusoids added (N_ac).

# In[7]:

time = np.arange(0, period_square, period_square/1000)
Eapp = np.arange(0, period_square, period_square/1000)
Es = E_start_new
Er = E_rev_new
vs = vscan_new
i = -1
for tt in time:
    i=i+1
    Eapp[i] = EvaluateRamp(Es, Er, vs, tt)
    EAC = 0.0
    for j in range(0,N_ac):
        EAC = EAC + EvaluateEAC(amp[j], freq[j], tt)
    Eapp[i] = Eapp[i] + EAC

plt.figure(figsize=(10,8), dpi=100)
plt.plot(time,Eapp)
plt.xlabel('Time (s)')
plt.ylabel('Voltage (V)')
plt.savefig('SquareWave.png')


# Output MECSim inputs
# ------

# In[8]:

print "Change the following variables in Master.inp:"
print E_start_new, "     ! E_start (V)"
print E_rev_new, "     ! E_rev (V)"
print vscan_new, "     ! scan speed (V/s)"
print
PrintSquareWave(amp,freq,N_ac)

